//**********************************************************************
// 	Project: 
//	File: gamma_exp.v
// 	Description: test GAMMA （GW_2A_18）
//	Author: wenxuan
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	|           	| xx/xx/2021	    | Initial version
//**********************************************************************


module gamma_exp(
// --- sym ---
	input    SYM_CLK,
// --- KEY ---
	input    KEY1,
// --- SW ----
	input    SW1,

	output			outvalid,
    output          line_sync_o,
    output          frame_sync_o,
	output [7:0]	R_dout,
	output [7:0]	G_dout,
	output [7:0]	B_dout
);

//**********************************************************************
// --- Parameter
//**********************************************************************
    localparam NUM_CYCLE = 3;
	localparam MEM_DEPTH = 8*8;
`define 	IMAGE_WIDTH 8	
//**********************************************************************
// --- Inter Signal Declaration
//**********************************************************************
    wire clk;
    // wire clk_100M;
	wire rstn;
	wire ce;
wire frame_sync;

    assign clk = SYM_CLK;
    assign rstn = KEY1;
	assign ce=SW1;

    wire						inpvalid;
    wire                        line_sync;
//	wire signed [17:0]	din;
	
	wire recycle;
	reg pstart;
	reg [19:0] addr;
    reg [9:0] cnt_cycle;

    reg [7:0] data_mem [0:MEM_DEPTH-1] /* synthesis syn_romstyle= "block_rom" */;
    reg [7:0] din;
//**********************************************************************
// --- Initial
//**********************************************************************

initial begin
           $readmemb("data.dat",data_mem);
end

assign inpvalid  = (addr ==1'b0) ? 1'b0 :(addr>=MEM_DEPTH+1) ? 1'b0 : (pstart && cnt_cycle == 0) ? 1'b1 : 1'b0 ;
assign line_sync = (addr>=MEM_DEPTH+1) ? 1'b0 : (pstart && addr[2:0] == 3'b1 && cnt_cycle == 0) ? 1'b1 : 1'b0 ;

assign frame_sync= (addr==1 && cnt_cycle==0 && line_sync) ? 1'b1: 1'b0;
assign recycle = (addr == 0 && cnt_cycle == NUM_CYCLE-2) ? 1'b1 : 1'b0;

    always @(posedge clk or negedge rstn) begin
        if(!rstn) cnt_cycle <= NUM_CYCLE-1;
        else if(ce) begin
			if(cnt_cycle == NUM_CYCLE-1 ) cnt_cycle <= 0;
            else if(cnt_cycle == NUM_CYCLE-1) cnt_cycle <= cnt_cycle;
            else cnt_cycle <= cnt_cycle + 1;
        end
    end

    always @(posedge clk or negedge rstn) begin
        if(!rstn) addr <= 0;
//        else if(ce & pstart) begin
        else if(ce) begin
			if(cnt_cycle == NUM_CYCLE-1 && addr == MEM_DEPTH+1+2*`IMAGE_WIDTH+3) addr <= 0;
            else if(cnt_cycle == NUM_CYCLE-1) addr <= addr + 1;
        end
    end


	always @(posedge clk or negedge rstn) begin
        if(!rstn) pstart <= 1'b0;
        else if(ce) begin
            if(cnt_cycle == NUM_CYCLE-1) pstart <= 1'b1;
        end
    end
	
    always @(posedge clk or negedge rstn) begin
        if(!rstn) din <= 0;
//        else if(ce & pstart) begin
        else if(ce ) begin
			if(cnt_cycle == NUM_CYCLE-1 && addr >= MEM_DEPTH) din <= 0;
            else if(cnt_cycle == NUM_CYCLE-1) din <= data_mem[addr];
        end
    end

//**********************************************************************
// --- Module: top 
// --- Description:	top module of project
//**********************************************************************
Gamma_Correction_Top Gamma_Correction_Top(
       .clk(clk), 				// system clock
       .rstn(rstn),				// reset
    .frame_sync (frame_sync),
    .line_sync (line_sync),
    .inpvalid (inpvalid),
    .R_din (din),
    .G_din (din),
    .B_din (din),
	.wr (1'b0), //input wr
	.waddr(16'b0), //input [15:0] waddr
	.wdata(32'b0), //input [31:0] wdata
    .frame_sync_o (frame_sync_o),
    .line_sync_o (line_sync_o),
    .outvalid(outvalid),
    .R_dout(R_dout),					// output data
    .G_dout(G_dout),					// output data
    .B_dout(B_dout)					// output data
);

	
 
endmodule
